/*** FRAGMENT.C ***/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dir.h>
#include <dos.h>

#define MAX 30000              /* MAX deve ser menor ou igual a 65535 */

#define MIN(x, y) ((x) < (y) ? ((unsigned) (x)) : ((unsigned) (y)))

char nome_arq[100];

/* ------------------------------------------------------------------------- */

int NPartes(char *nome_origem, long lim) {

  FILE *origem;
  int quant;
  long tam_origem;
  struct ffblk fbloco;

  findfirst(nome_origem, &fbloco, FA_ARCH);
  tam_origem = fbloco.ff_fsize;
  quant = tam_origem/lim;
  while (tam_origem > lim*quant) quant++;  
  return quant;
}

/* ------------------------------------------------------------------------- */

void FragmentaArquivo(char *nome_origem, char *nome_base, int n_partes) {

  FILE *origem, *destino;
  char nome_destino[100], temp[MAX];

  int k;
  long tam_parte, tam_origem, total_lidos, ultima_parte;
  struct ffblk fbloco;
  unsigned max_lidos, n_lidos;

  if ((origem = fopen(nome_origem, "rb")) == NULL) {
    printf("Arquivo %s nao pode ser aberto\n", nome_origem);
    return;
  }

  findfirst(nome_origem, &fbloco, FA_ARCH);
  tam_origem = fbloco.ff_fsize;
  tam_parte = tam_origem/n_partes;
  ultima_parte = tam_origem - (n_partes - 1)*tam_parte;

  printf("\nFragmentando o arquivo %s", nome_origem);
  for (k = 1; k <= n_partes; k++) {

    /* Inicializa o total de bytes lidos e define o nome e abre o
    arquivo parcial a ser criado */
    total_lidos = 0;
    sprintf(nome_destino, "%s.%d", nome_base, k);
    printf("\n   %c Criando %s", 254, strupr(nome_destino));
    if ((destino = fopen(nome_destino, "wb")) == NULL) {
      printf("Arquivo %s nao pode ser aberto\n", nome_destino);
      return;
    }

    /* Le no arquivo fonte e escreve no arquivo parcial */
    max_lidos = MIN(MAX, tam_parte);
    do {
      n_lidos = fread(temp, sizeof(char), max_lidos, origem);

      total_lidos += n_lidos;

      if (k < n_partes)
	max_lidos = MIN(MAX, tam_parte - total_lidos);
      else
	max_lidos = MIN(MAX, ultima_parte - total_lidos);


      fwrite(temp, sizeof(char), n_lidos, destino);
    } while (n_lidos > 0 && ((k < n_partes && total_lidos < tam_parte)
			 || (k == n_partes && total_lidos < ultima_parte)));

    /* Fecha o arquivo parcial aberto */
    fclose(destino);
  }

  fclose(origem);
}

/* ------------------------------------------------------------------------- */

void CriaArquivoBAT(char *nome_origem, char *nome_base, int n_partes) {

  FILE *bat;
  int k;
  char comando[1000], comando_aux[1000], arquivo_bat[200];

  /* Cria o arquivo *.BAT para emendar as partes posteriormente */

  sprintf(arquivo_bat, "%s.bat", nome_base);
  if ((bat = fopen(arquivo_bat, "wt")) == NULL) return;
  printf("\n\nCriando o arquivo %s\n", arquivo_bat);
  fprintf(bat, "@echo off\n");
  strcpy(comando, "COPY /B ");
  for (k = 1; k <= n_partes; k++) {
    if (k < n_partes)
      sprintf(comando_aux, "%s.%d+", nome_base, k);
    else
      sprintf(comando_aux, "%s.%d %s", nome_base, k, nome_origem);
    strcat(comando, comando_aux);
  }
  fprintf(bat, "%s\n", comando);
  fprintf(bat, "echo.\necho Arquivo %s restaurado\necho.\n", nome_origem); 
  fclose(bat);
  printf("\nPara restaurar o arquivo original digite %s\n",
				   strupr(nome_base));
}

/* ------------------------------------------------------------------------- */

void main(int argc, char **argv) {

  char nome_origem[200], nome_base[200], n_partes_str[50];
  int n_partes, sug;
  long lim = 1457664L;

  printf("\n------------------- FRAGMENT.COM ----------------------");
  printf("\n  Fragmenta arquivos em varias partes de mesmo tamanho ");  
  printf("\n(C) 2000, Lenimar Nunes de Andrade, lenimar@mat.ufpb.br");
  printf("\n-------------------------------------------------------\n");

  if (argc > 1)
    strcpy(nome_origem, argv[1]);
  else {
    printf("\nNome do arquivo a ser fragmentado: ");
    gets(nome_origem);
  }
  strcpy(nome_arq, nome_origem);

  if (argc > 2)
    strcpy(nome_base, argv[2]);
  else {
    printf("\nNome base (max. 6 caracteres) : ");
    gets(nome_base);
  }
  nome_base[6] = '\0';

  if (argc > 3) {
    strcpy(n_partes_str, argv[3]);
    n_partes = atoi(n_partes_str);
  } else {
    sug = NPartes(nome_origem, lim);
    if (sug > 1)
      printf("\nNumero de partes [Sugestao: %d] : ",NPartes(nome_origem, lim));
    else
      printf("\nNumero de partes: ");
    scanf("%d", &n_partes);
  }

  FragmentaArquivo(nome_origem, nome_base, n_partes);

  CriaArquivoBAT(nome_origem, nome_base, n_partes);

}

/* ------------------------------------------------------------------------- */

/*** FIM DE FRAGMENT.C ***/
